/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.ddv.converter;

import dk.sundhedsdatastyrelsen.cbs.dto.vaccination.EffectuatedVaccination;
import dk.vaccinationsregister.schemas._2021._04._01.EffectuatedItemType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.convert.converter.Converter;
import org.springframework.lang.NonNull;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Component;

import javax.xml.datatype.XMLGregorianCalendar;
import java.time.LocalDateTime;

import static dk.sundhedsdatastyrelsen.cbs.dto.impl.vaccination.EffectuatedVaccinationBuilder.effectuatedVaccination;
import static java.util.Objects.requireNonNull;

@Component
public class EffectuatedVaccinationConverter implements Converter<EffectuatedItemType, EffectuatedVaccination> {

  @NonNull
  private final Converter<XMLGregorianCalendar, LocalDateTime> converter;

  @Autowired
  public EffectuatedVaccinationConverter(@NonNull final Converter<XMLGregorianCalendar, LocalDateTime> converter) {
    this.converter = converter;
  }

  @Override
  @Nullable
  public EffectuatedVaccination convert(@NonNull final EffectuatedItemType source) {
    final XMLGregorianCalendar calendar = source.getEffectuatedVaccinationDateTime();
    if (calendar == null) return null;

    return effectuatedVaccination(source.getEffectuatedVaccineName(), requireNonNull(converter.convert(calendar)))
      .vaccineIdentifier(source.getEffectuatedVaccineIdentifier())
      .vaccinationIdentifier(source.getEffectuatedVaccinationIdentifier())
      .vaccinationVersionIdentifier(source.getEffectuatedVaccinationVersionIdentifier())
      .build();
  }
}
