/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package COSE;

import com.upokecenter.cbor.CBORObject;
import com.upokecenter.cbor.CBORType;

/**
 *
 * @author Jim
 */
public class CounterSign1 extends Signer {
    public CounterSign1()
    {
        contextString = "CounterSignature0";
    }
    
    public CounterSign1(byte[] rgb) {
        contextString = "CounterSignature0";
        rgbSignature = rgb;
        rgbProtected = new byte[0];
    }
    
    public CounterSign1(OneKey key) {
        super(key);
        contextString = "CounterSignature0";
        objUnprotected.Clear();
        objProtected.Clear();
    }
    
    private Message m_msgToSign;
    private Signer m_signerToSign;
    
    public void setObject(Message msg)
    {
        m_msgToSign = msg;
    }
    
    public void setObject(Signer signer)
    {
        m_signerToSign = signer;
    }
    
    public void setKey(OneKey key)
    {
        cnKey = key;
    }
    
    @Override
    public void DecodeFromCBORObject(CBORObject cbor) throws CoseException {
        if (cbor.getType() != CBORType.ByteString) {
            throw new CoseException("Invalid format for Countersignature0");
        }
        
        rgbSignature = cbor.GetByteString();
        rgbProtected = new byte[0];
    }
    
    public CBORObject EncodeToCBORObject() throws CoseException {
        if (!objProtected.getValues().isEmpty() || !objUnprotected.getValues().isEmpty()) {
            throw new CoseException("CounterSign1 object cannot have protected or unprotected attributes");
        }
        
        return CBORObject.FromObject(rgbSignature);
    }
            
}
