/*
 * The MIT License
 *
 * Original work sponsored and donated by The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 * Copyright (C) 2021 The Danish Health Data Authority (http://www.sundhedsdatastyrelsen.dk)
 *
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is furnished to do
 * so, subject to the following conditions:
 *
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package dk.sundhedsdatastyrelsen.cbs.test;

import org.springframework.lang.NonNull;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Random;
import java.util.function.Supplier;
import java.util.stream.Collectors;

import static java.lang.String.valueOf;
import static java.nio.charset.StandardCharsets.UTF_8;
import static java.time.LocalDate.ofYearDay;
import static java.time.format.DateTimeFormatter.ofPattern;
import static java.util.stream.Stream.generate;
import static org.apache.commons.lang3.StringUtils.leftPad;

public class RandomData extends Random {

  private static final DateTimeFormatter CPR_FORMATTER = ofPattern("ddMMyy");

  public RandomData(final long seed) {
    super(seed);
  }

  @NonNull
  public String cpr() {
    return ofYearDay(nextInt(100) + 1900, nextInt(365) + 1)
      .format(CPR_FORMATTER) + leftPad(valueOf(nextInt(10000)), 4, '0');
  }

  @NonNull
  public <T> List<T> list(final int bound, @NonNull final Supplier<T> constructor) {
    return generate(constructor).limit(nextInt(bound)).collect(Collectors.toList());
  }

  @NonNull
  public XMLGregorianCalendar xmlGregorianCalendar() {
    final GregorianCalendar calendar = new GregorianCalendar();
    calendar.setTime(new Date());
    try {
      final DatatypeFactory factory = DatatypeFactory.newInstance();
      return factory.newXMLGregorianCalendar(calendar);
    } catch (DatatypeConfigurationException e) {
      throw new RuntimeException(e);
    }
  }

  @NonNull
  public String string(final int maxLength) {
    final byte[] bytes = new byte[nextInt(maxLength)];
    nextBytes(bytes);
    return new String(bytes, UTF_8);
  }
}
